@@ -14,7 +14,7 @@ use std::{
 };
 
 use byteorder::{LittleEndian, ReadBytesExt};
-use noodles_bgzf::{self as bgzf, VirtualPosition};
+use noodles_bgzf as bgzf;
 use noodles_core::{region::Interval, Region};
 use noodles_csi::{BinningIndex, BinningIndexReferenceSequence};
 use noodles_sam::header::{ReferenceSequence, ReferenceSequences};
@@ -98,15 +98,7 @@ where
     /// # Ok::<(), io::Error>(())
     /// ```
     pub fn read_header(&mut self) -> io::Result<String> {
-        let magic = read_magic(&mut self.inner)?;
-
-        if magic != MAGIC_NUMBER {
-            return Err(io::Error::new(
-                io::ErrorKind::InvalidData,
-                "invalid BAM header",
-            ));
-        }
-
+        read_magic(&mut self.inner)?;
         read_header(&mut self.inner)
     }
 
@@ -132,18 +124,7 @@ where
     /// # Ok::<(), io::Error>(())
     /// ```
     pub fn read_reference_sequences(&mut self) -> io::Result<ReferenceSequences> {
-        let n_ref = self.inner.read_u32::<LittleEndian>().and_then(|n| {
-            usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
-        })?;
-
-        let mut reference_sequences = ReferenceSequences::with_capacity(n_ref);
-
-        for _ in 0..n_ref {
-            let reference_sequence = read_reference_sequence(&mut self.inner)?;
-            reference_sequences.insert(reference_sequence.name().into(), reference_sequence);
-        }
-
-        Ok(reference_sequences)
+        read_reference_sequences(&mut self.inner)
     }
 
     /// Reads a single record.
@@ -177,18 +158,7 @@ where
     /// # Ok::<(), io::Error>(())
     /// ```
     pub fn read_record(&mut self, record: &mut Record) -> io::Result<usize> {
-        let block_size = match self.inner.read_u32::<LittleEndian>() {
-            Ok(bs) => {
-                usize::try_from(bs).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?
-            }
-            Err(ref e) if e.kind() == io::ErrorKind::UnexpectedEof => return Ok(0),
-            Err(e) => return Err(e),
-        };
-
-        record.resize(block_size);
-        self.inner.read_exact(record)?;
-
-        Ok(block_size)
+        read_record(&mut self.inner, record)
     }
 
     /// Returns an iterator over records starting from the current stream position.
@@ -232,7 +202,7 @@ where
     /// assert_eq!(virtual_position.uncompressed(), 0);
     /// # Ok::<(), io::Error>(())
     /// ```
-    pub fn virtual_position(&self) -> VirtualPosition {
+    pub fn virtual_position(&self) -> bgzf::VirtualPosition {
         self.inner.virtual_position()
     }
 }
@@ -256,14 +226,14 @@ where
     /// reader.seek(virtual_position)?;
     /// # Ok::<(), io::Error>(())
     /// ```
-    pub fn seek(&mut self, pos: VirtualPosition) -> io::Result<VirtualPosition> {
+    pub fn seek(&mut self, pos: bgzf::VirtualPosition) -> io::Result<bgzf::VirtualPosition> {
         self.inner.seek(pos)
     }
 
     // Seeks to the first record by setting the cursor to the beginning of the stream and
     // (re)reading the header and binary reference sequences.
-    fn seek_to_first_record(&mut self) -> io::Result<VirtualPosition> {
-        self.seek(VirtualPosition::default())?;
+    fn seek_to_first_record(&mut self) -> io::Result<bgzf::VirtualPosition> {
+        self.seek(bgzf::VirtualPosition::default())?;
         self.read_header()?;
         self.read_reference_sequences()?;
         Ok(self.virtual_position())
@@ -337,13 +307,21 @@ where
     }
 }
 
-fn read_magic<R>(reader: &mut R) -> io::Result<[u8; 4]>
+fn read_magic<R>(reader: &mut R) -> io::Result<()>
 where
     R: Read,
 {
     let mut magic = [0; 4];
     reader.read_exact(&mut magic)?;
-    Ok(magic)
+
+    if magic == MAGIC_NUMBER {
+        Ok(())
+    } else {
+        Err(io::Error::new(
+            io::ErrorKind::InvalidData,
+            "invalid BAM header",
+        ))
+    }
 }
 
 fn read_header<R>(reader: &mut R) -> io::Result<String>
@@ -364,6 +342,24 @@ where
     })
 }
 
+fn read_reference_sequences<R>(reader: &mut R) -> io::Result<ReferenceSequences>
+where
+    R: Read,
+{
+    let n_ref = reader.read_u32::<LittleEndian>().and_then(|n| {
+        usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
+    })?;
+
+    let mut reference_sequences = ReferenceSequences::with_capacity(n_ref);
+
+    for _ in 0..n_ref {
+        let reference_sequence = read_reference_sequence(reader)?;
+        reference_sequences.insert(reference_sequence.name().into(), reference_sequence);
+    }
+
+    Ok(reference_sequences)
+}
+
 fn read_reference_sequence<R>(reader: &mut R) -> io::Result<ReferenceSequence>
 where
     R: Read,
@@ -383,6 +379,22 @@ where
     ReferenceSequence::new(name, l_ref).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
 }
 
+fn read_record<R>(reader: &mut R, record: &mut Record) -> io::Result<usize>
+where
+    R: Read,
+{
+    let block_size = match reader.read_u32::<LittleEndian>() {
+        Ok(bs) => usize::try_from(bs).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?,
+        Err(ref e) if e.kind() == io::ErrorKind::UnexpectedEof => return Ok(0),
+        Err(e) => return Err(e),
+    };
+
+    record.resize(block_size);
+    reader.read_exact(record)?;
+
+    Ok(block_size)
+}
+
 pub(crate) fn bytes_with_nul_to_string(buf: &[u8]) -> io::Result<String> {
     CStr::from_bytes_with_nul(buf)
         .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
@@ -417,3 +429,86 @@ pub(crate) fn resolve_region(
         ))
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_read_magic() -> io::Result<()> {
+        let data = b"BAM\x01";
+        let mut reader = &data[..];
+        assert!(read_magic(&mut reader).is_ok());
+
+        let data = [];
+        let mut reader = &data[..];
+        assert!(matches!(
+            read_magic(&mut reader),
+            Err(ref e) if e.kind() == io::ErrorKind::UnexpectedEof
+        ));
+
+        let data = b"MThd";
+        let mut reader = &data[..];
+        assert!(matches!(
+            read_magic(&mut reader),
+            Err(ref e) if e.kind() == io::ErrorKind::InvalidData
+        ));
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_read_header() -> io::Result<()> {
+        let expected = "@HD\tVN:1.6\n";
+
+        let data_len = expected.len() as u32;
+        let mut data = data_len.to_le_bytes().to_vec();
+        data.extend(expected.as_bytes());
+
+        let mut reader = &data[..];
+        let actual = read_header(&mut reader)?;
+
+        assert_eq!(actual, expected);
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_read_reference_sequences() -> Result<(), Box<dyn std::error::Error>> {
+        let data = [
+            0x01, 0x00, 0x00, 0x00, // n_ref = 1
+            0x04, 0x00, 0x00, 0x00, // ref[0].l_name = 4
+            0x73, 0x71, 0x30, 0x00, // ref[0].name = "sq0\x00"
+            0x08, 0x00, 0x00, 0x00, // ref[0].l_ref = 8
+        ];
+
+        let mut reader = &data[..];
+        let actual = read_reference_sequences(&mut reader)?;
+
+        let expected: ReferenceSequences = vec![("sq0", 8)]
+            .into_iter()
+            .map(|(name, len)| ReferenceSequence::new(name, len).map(|rs| (name.into(), rs)))
+            .collect::<Result<_, _>>()?;
+
+        assert_eq!(actual, expected);
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_read_record() -> io::Result<()> {
+        let data = [
+            0x04, 0x00, 0x00, 0x00, // block_size = 4
+            0x6e, 0x64, 0x6c, 0x73, // ...
+        ];
+
+        let mut reader = &data[..];
+        let mut record = Record::default();
+        let block_size = read_record(&mut reader, &mut record)?;
+
+        assert_eq!(block_size, 4);
+        assert_eq!(&record[..], &data[4..]);
+
+        Ok(())
+    }
+}
