@@ -13,7 +13,7 @@ use noodles_sam::{
     header::{ReferenceSequence, ReferenceSequences},
 };
 
-use super::{Record, MAGIC_NUMBER};
+use super::Record;
 
 /// A BAM writer.
 ///
@@ -111,16 +111,7 @@ where
     /// # Ok::<(), io::Error>(())
     /// ```
     pub fn write_header(&mut self, header: &sam::Header) -> io::Result<()> {
-        self.inner.write_all(MAGIC_NUMBER)?;
-
-        let text = header.to_string();
-        let l_text = i32::try_from(text.len())
-            .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
-        self.inner.write_i32::<LittleEndian>(l_text)?;
-
-        self.inner.write_all(text.as_bytes())?;
-
-        Ok(())
+        write_header(&mut self.inner, header)
     }
 
     /// Writes SAM reference sequences.
@@ -149,15 +140,7 @@ where
         &mut self,
         reference_sequences: &ReferenceSequences,
     ) -> io::Result<()> {
-        let n_ref = i32::try_from(reference_sequences.len())
-            .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
-        self.inner.write_i32::<LittleEndian>(n_ref)?;
-
-        for reference_sequence in reference_sequences.values() {
-            write_reference_sequence(&mut self.inner, reference_sequence)?;
-        }
-
-        Ok(())
+        write_reference_sequences(&mut self.inner, reference_sequences)
     }
 
     /// Writes a BAM record.
@@ -173,10 +156,7 @@ where
     /// # Ok::<(), io::Error>(())
     /// ```
     pub fn write_record(&mut self, record: &Record) -> io::Result<()> {
-        let block_size = u32::try_from(record.len())
-            .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
-        self.inner.write_u32::<LittleEndian>(block_size)?;
-        self.inner.write_all(record)
+        write_record(&mut self.inner, record)
     }
 
     /// Writes a SAM record.
@@ -204,6 +184,42 @@ where
     }
 }
 
+fn write_header<W>(writer: &mut W, header: &sam::Header) -> io::Result<()>
+where
+    W: Write,
+{
+    use super::MAGIC_NUMBER;
+
+    writer.write_all(MAGIC_NUMBER)?;
+
+    let text = header.to_string();
+    let l_text =
+        i32::try_from(text.len()).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+    writer.write_i32::<LittleEndian>(l_text)?;
+
+    writer.write_all(text.as_bytes())?;
+
+    Ok(())
+}
+
+fn write_reference_sequences<W>(
+    writer: &mut W,
+    reference_sequences: &ReferenceSequences,
+) -> io::Result<()>
+where
+    W: Write,
+{
+    let n_ref = i32::try_from(reference_sequences.len())
+        .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+    writer.write_i32::<LittleEndian>(n_ref)?;
+
+    for reference_sequence in reference_sequences.values() {
+        write_reference_sequence(writer, reference_sequence)?;
+    }
+
+    Ok(())
+}
+
 fn write_reference_sequence<W>(
     writer: &mut W,
     reference_sequence: &ReferenceSequence,
@@ -226,6 +242,19 @@ where
     Ok(())
 }
 
+fn write_record<W>(writer: &mut W, record: &Record) -> io::Result<()>
+where
+    W: Write,
+{
+    let block_size =
+        u32::try_from(record.len()).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+    writer.write_u32::<LittleEndian>(block_size)?;
+
+    writer.write_all(record)?;
+
+    Ok(())
+}
+
 #[cfg(test)]
 mod tests {
     use std::convert::TryFrom;
@@ -240,46 +269,58 @@ mod tests {
     fn test_write_header() -> io::Result<()> {
         use sam::header::header::{Header, Version};
 
-        let mut writer = Writer::new(Vec::new());
-
         let header = sam::Header::builder()
             .set_header(Header::new(Version::new(1, 6)))
             .build();
 
-        writer.write_header(&header)?;
-        writer.try_finish()?;
+        let mut buf = Vec::new();
+        write_header(&mut buf, &header)?;
 
-        let mut reader = Reader::new(writer.get_ref().as_slice());
-        let actual = reader.read_header()?;
+        let mut expected = vec![
+            b'B', b'A', b'M', 0x01, // magic
+            0x0b, 0x00, 0x00, 0x00, // l_text = 11
+        ];
+        expected.extend_from_slice(b"@HD\tVN:1.6\n"); // text
 
-        let expected = "@HD\tVN:1.6\n";
-
-        assert_eq!(actual, expected);
+        assert_eq!(buf, expected);
 
         Ok(())
     }
 
     #[test]
     fn test_write_reference_sequences() -> Result<(), Box<dyn std::error::Error>> {
-        let mut writer = Writer::new(Vec::new());
+        let reference_sequences = vec![("sq0", 8)]
+            .into_iter()
+            .map(|(name, len)| ReferenceSequence::new(name, len).map(|rs| (name.into(), rs)))
+            .collect::<Result<_, _>>()?;
 
-        let reference_sequence = ReferenceSequence::new("sq0", 8)?;
+        let mut buf = Vec::new();
+        write_reference_sequences(&mut buf, &reference_sequences)?;
 
-        let header = sam::Header::builder()
-            .add_reference_sequence(reference_sequence.clone())
-            .set_header(sam::header::header::Header::default())
-            .build();
+        let expected = [
+            0x01, 0x00, 0x00, 0x00, // n_ref = 1
+            0x04, 0x00, 0x00, 0x00, // ref[0].l_name = 4
+            b's', b'q', b'0', 0x00, // ref[0].name = b"sq0\x00"
+            0x08, 0x00, 0x00, 0x00, // ref[0].l_ref = 8
+        ];
 
-        writer.write_header(&header)?;
-        writer.write_reference_sequences(header.reference_sequences())?;
-        writer.try_finish()?;
+        assert_eq!(buf, expected);
 
-        let mut reader = Reader::new(writer.get_ref().as_slice());
-        reader.read_header()?;
-        let actual = reader.read_reference_sequences()?;
+        Ok(())
+    }
 
-        assert_eq!(actual.len(), 1);
-        assert_eq!(&actual[0], &reference_sequence);
+    #[test]
+    fn test_write_record() -> Result<(), Box<dyn std::error::Error>> {
+        let record = Record::default();
+
+        let mut buf = Vec::new();
+        write_record(&mut buf, &record)?;
+
+        let block_size = u32::try_from(record.len())?;
+        let mut expected = block_size.to_le_bytes().to_vec();
+        expected.extend_from_slice(&record);
+
+        assert_eq!(buf, expected);
 
         Ok(())
     }
