@@ -20,9 +20,6 @@ use crate::record::sequence::Base;
 // (4) + next_ref_id (4) + next_pos (4) + tlen (4)
 const BLOCK_HEADER_SIZE: u32 = 32;
 
-// § 4.2.1 BIN field calculation (2021-06-03)
-const UNMAPPED_BIN: u16 = 4680;
-
 // § 4.2.3 SEQ and QUAL encoding (2021-06-03)
 const NULL_QUALITY_SCORE: u8 = 255;
 
@@ -72,25 +69,16 @@ where
 
     writer.write_u8(l_read_name)?;
 
-    let mapq = u8::from(record.mapping_quality());
-    writer.write_u8(mapq)?;
+    // mapq
+    write_mapping_quality(writer, record.mapping_quality())?;
 
-    let bin = record
-        .position()
-        .map(|v| i32::from(v) - 1)
-        .map(|start| {
-            // 0-based, [start, end)
-            let reference_len = record.cigar().reference_len() as i32;
-            let end = start + reference_len;
-            region_to_bin(start, end) as u16
-        })
-        .unwrap_or(UNMAPPED_BIN);
-    writer.write_u16::<LittleEndian>(bin)?;
+    // bin
+    write_bin(writer, record)?;
 
     writer.write_u16::<LittleEndian>(n_cigar_op)?;
 
-    let flag = u16::from(record.flags());
-    writer.write_u16::<LittleEndian>(flag)?;
+    // flag
+    write_flags(writer, record.flags())?;
 
     writer.write_u32::<LittleEndian>(l_seq)?;
 
@@ -104,8 +92,8 @@ where
     // next_pos
     write_position(writer, record.mate_position())?;
 
-    let tlen = record.template_length();
-    writer.write_i32::<LittleEndian>(tlen)?;
+    // tlen
+    write_template_length(writer, record.template_length())?;
 
     writer.write_all(read_name)?;
 
@@ -182,6 +170,53 @@ where
     writer.write_i32::<LittleEndian>(pos)
 }
 
+fn write_mapping_quality<W>(
+    writer: &mut W,
+    mapping_quality: sam::record::MappingQuality,
+) -> io::Result<()>
+where
+    W: Write,
+{
+    let mapq = u8::from(mapping_quality);
+    writer.write_u8(mapq)
+}
+
+fn write_bin<W>(writer: &mut W, record: &sam::Record) -> io::Result<()>
+where
+    W: Write,
+{
+    // § 4.2.1 "BIN field calculation" (2021-06-03): "Note unmapped reads with `POS` 0 (which
+    // becomes -1 in BAM) therefore use `reg2bin(-1, 0)` which is computed as 4680."
+    const UNMAPPED_BIN: u16 = 4680;
+
+    let bin = record
+        .position()
+        .map(|p| i32::from(p) - 1)
+        .map(|start| {
+            let reference_len = record.cigar().reference_len() as i32;
+            let end = start + reference_len;
+            region_to_bin(start, end) as u16
+        })
+        .unwrap_or(UNMAPPED_BIN);
+
+    writer.write_u16::<LittleEndian>(bin)
+}
+
+fn write_flags<W>(writer: &mut W, flags: sam::record::Flags) -> io::Result<()>
+where
+    W: Write,
+{
+    let flag = u16::from(flags);
+    writer.write_u16::<LittleEndian>(flag)
+}
+
+fn write_template_length<W>(writer: &mut W, tlen: i32) -> io::Result<()>
+where
+    W: Write,
+{
+    writer.write_i32::<LittleEndian>(tlen)
+}
+
 fn write_cigar<W>(writer: &mut W, cigar: &Cigar) -> io::Result<()>
 where
     W: Write,
